from typing import List

from pydantic import BaseModel

from .StatusMessage import StatusMessage


class Failure(BaseModel):
    """
    Details of a failed task

    Attributes
    ----------
    errors: List[StatusMessage]: list of status messages describing failure
    """

    errors: List[StatusMessage]



from typing import Dict, List

from pydantic import BaseModel


class StatusMessage(BaseModel):
    """
    Represents a single status message, in a form amenable to internationalisation.

    Each message contains a code, which can be looked up in a list to find the actual text in any of the available
    languages.  The text can contain numbered placeholders of the form <code>{0}</code>, which are filled in with values
    specified in the "params" property. The "text" property provides a single fallback text to be used if the specified
    code cannot be found in the lookup table.

    Attributes
    ----------
    code: (str, required) status code to be found in lookup table
    params: (List[str], required) values to fill in message placeholder
    text: (str, requored) fallback text to be used if specified code cannot be found in lookup table
    detail: (Dict, optional) arbitrary further details that don't need translation (e.g: stacktrace)
    """

    code: str
    params: List[str]
    text: str
    detail: Dict = None



import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseELGCommand


def info_command_factory(args):
    return InfoCommand(
        id=args.id,
        domain=args.domain,
        not_use_cache=args.not_use_cache,
        cache_dir=args.cache_dir,
    )


class InfoCommand(BaseELGCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        info_parser = parser.add_parser("info", description="Get information of an ELG entity")
        info_parser.add_argument(
            "--id", type=int, default=None, required=True, help="ID of the entity to obtain info on"
        )
        info_parser.add_argument(
            "--domain",
            type=str,
            default="live",
            required=False,
            help="ELG domain you want to use",
        )
        info_parser.add_argument(
            "--not_use_cache",
            default=False,
            action="store_true",
            required=False,
            help="use to not use cache",
        )
        info_parser.add_argument(
            "--cache_dir", type=str, default="~/.cache/elg", required=False, help="path to the cache directory"
        )
        info_parser.set_defaults(func=info_command_factory)

    def __init__(
        self,
        id: int = None,
        domain: str = "live",
        not_use_cache: bool = False,
        cache_dir: str = "~/.cache/elg",
    ):
        self._id = id
        self._domain = domain
        self._use_cache = not not_use_cache
        self._cache_dir = cache_dir

    def run(self):
        from ..entity import Entity

        try:
            entity = Entity.from_id(
                id=self._id,
                domain=self._domain,
                use_cache=self._use_cache,
                cache_dir=self._cache_dir,
            )
            logger.info(f"Get info of {entity.resource_name}")
            print(entity)
        except Exception as e:
            logger.error(f"Error during corpus info - {e}")
            sys.exit(1)



__version__ = "0.4.5"

from .authentication import Authentication
from .benchmark import Benchmark
from .catalog import Catalog
from .corpus import Corpus
from .entity import Entity
from .flask_service import FlaskService
from .pipeline import Pipeline
from .provider import Provider
from .service import Service



from typing import Dict, List

from pydantic import BaseModel


class Annotation(BaseModel):
    """
    Representation of a single annotation with respect to either one or two streams of data.

    Attributes
    ----------
    start (int, required): annotation start location (in response)
    end (int, required): annotation end location (in response)
    source_start (int, required in cases): annotation start location (in source)
    source_end (int, required in cases): annotation end location (in source)
    features (dict, optional): arbitrary json metadata about content
    """

    start: int
    end: int
    sourceStart: int = None
    sourceEnd: int = None
    features: dict = None



import sys
from argparse import ArgumentParser
from typing import List, Union

from loguru import logger

from . import BaseELGCommand


def docker_create_command_factory(args):
    return DockerCreateCommand(
        classname=args.classname,
        path=args.path,
        requirements=args.requirements,
        requirements_file=args.requirements_file,
        required_files=args.required_files,
        required_folders=args.required_folders,
    )


class DockerCreateCommand(BaseELGCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        info_parser = parser.add_parser("create", description="Create requirements.txt and Docker files.")
        info_parser.add_argument(
            "-n", "--classname", type=str, default=None, required=True, help="Name of the Service Class"
        )
        info_parser.add_argument(
            "-p",
            "--path",
            type=str,
            default="",
            required=None,
            help="Path the python script containing the Service Class definition from the current location",
        )
        info_parser.add_argument(
            "-r",
            "--requirements",
            type=str,
            action="append",
            default=None,
            required=False,
            help="Pypi requirements for the Service Class, e.g. `nltk==3.5`",
        )
        info_parser.add_argument(
            "--requirements_file",
            type=str,
            default=None,
            required=False,
            help="Pypi requirements file, e.g. `requirements.txt`",
        )
        info_parser.add_argument(
            "-f",
            "--required_files",
            type=str,
            action="append",
            default=None,
            required=False,
            help="Name of the files from the directory of the Service Class",
        )
        info_parser.add_argument(
            "--required_folders",
            type=str,
            action="append",
            default=None,
            required=False,
            help="Name of the folders from the directory of the Service Class",
        )

        info_parser.set_defaults(func=docker_create_command_factory)

    def __init__(
        self,
        classname: str = None,
        path: str = "",
        requirements: Union[str, List[str]] = None,
        requirements_file: str = None,
        required_files: Union[str, List[str]] = None,
        required_folders: Union[str, List[str]] = None,
    ):
        self._classname = classname
        self._path = path
        self._requirements = requirements
        self._requirements_file = requirements_file
        self._required_files = required_files
        self._required_folders = required_folders

    def run(self):
        from ..flask_service import FlaskService

        try:
            logger.info(f"Creation of `requirements.txt`")
            if self._requirements is None:
                requirements = []
            elif isinstance(self._requirements, str):
                requirements = [self._requirements]
            else:
                requirements = self._requirements
            if self._requirements_file is None:
                pass
            else:
                with open(self._requirements_file) as f:
                    requirements += f.read().split("\n")
            FlaskService.create_requirements(requirements=requirements, path=self._path)
            logger.info(f"Creation of `Dockerfile` and `docker-entrypoint.sh`")
            if self._required_files is None:
                required_files = []
            elif isinstance(self._required_files, str):
                required_files = [self._required_files]
            else:
                required_files = self._required_files
            if self._required_folders is None:
                required_folders = []
            elif isinstance(self._required_folders, str):
                required_folders = [self._required_folders]
            else:
                required_folders = self._required_folders
            FlaskService.create_docker_files(
                required_files=required_files, required_folders=required_folders, path=self._path
            )
        except Exception as e:
            logger.error(f"Error during the creation of docker files - {e}")
            sys.exit(1)



data = """\
.aac 	AAC audio 	audio/aac
.abw 	AbiWord document 	application/x-abiword
.arc 	Archive document (multiple files embedded) 	application/x-freearc
.avi 	AVI: Audio Video Interleave 	video/x-msvideo
.azw 	Amazon Kindle eBook format 	application/vnd.amazon.ebook
.bin 	Any kind of binary data 	application/octet-stream
.bmp 	Windows OS/2 Bitmap Graphics 	image/bmp
.bz 	BZip archive 	application/x-bzip
.bz2 	BZip2 archive 	application/x-bzip2
.csh 	C-Shell script 	application/x-csh
.css 	Cascading Style Sheets (CSS) 	text/css
.csv 	Comma-separated values (CSV) 	text/csv
.doc 	Microsoft Word 	application/msword
.docx 	Microsoft Word (OpenXML) 	application/vnd.openxmlformats-officedocument.wordprocessingml.document
.eot 	MS Embedded OpenType fonts 	application/vnd.ms-fontobject
.epub 	Electronic publication (EPUB) 	application/epub+zip
.gz 	GZip Compressed Archive 	application/gzip
.gif 	Graphics Interchange Format (GIF) 	image/gif
.htm	HyperText Markup Language (HTML) 	text/html
.html 	HyperText Markup Language (HTML) 	text/html
.ico 	Icon format 	image/vnd.microsoft.icon
.ics 	iCalendar format 	text/calendar
.jar 	Java Archive (JAR) 	application/java-archive
.jpeg 	JPEG images 	image/jpeg
.jpg 	JPEG images 	image/jpeg
.js 	JavaScript 	text/javascript
.json 	JSON format 	application/json
.jsonld 	JSON-LD format 	application/ld+json
.mid 	Musical Instrument Digital Interface (MIDI) 	audio/midi audio/x-midi
.midi 	Musical Instrument Digital Interface (MIDI) 	audio/midi audio/x-midi
.mjs 	JavaScript module 	text/javascript
.mp3 	MP3 audio 	audio/mpeg
.mpeg 	MPEG Video 	video/mpeg
.mpkg 	Apple Installer Package 	application/vnd.apple.installer+xml
.odp 	OpenDocument presentation document 	application/vnd.oasis.opendocument.presentation
.ods 	OpenDocument spreadsheet document 	application/vnd.oasis.opendocument.spreadsheet
.odt 	OpenDocument text document 	application/vnd.oasis.opendocument.text
.oga 	OGG audio 	audio/ogg
.ogv 	OGG video 	video/ogg
.ogx 	OGG 	application/ogg
.opus 	Opus audio 	audio/opus
.otf 	OpenType font 	font/otf
.png 	Portable Network Graphics 	image/png
.pdf 	Adobe Portable Document Format (PDF) 	application/pdf
.php 	Hypertext Preprocessor (Personal Home Page) 	application/x-httpd-php
.ppt 	Microsoft PowerPoint 	application/vnd.ms-powerpoint
.pptx 	Microsoft PowerPoint (OpenXML) 	application/vnd.openxmlformats-officedocument.presentationml.presentation
.rar 	RAR archive 	application/vnd.rar
.rtf 	Rich Text Format (RTF) 	application/rtf
.sh 	Bourne shell script 	application/x-sh
.svg 	Scalable Vector Graphics (SVG) 	image/svg+xml
.swf 	Small web format (SWF) or Adobe Flash document 	application/x-shockwave-flash
.tar 	Tape Archive (TAR) 	application/x-tar
.tif 	Tagged Image File Format (TIFF) 	image/tiff
.tiff 	Tagged Image File Format (TIFF) 	image/tiff
.ts 	MPEG transport stream 	video/mp2t
.ttf 	TrueType Font 	font/ttf
.txt 	Text, (generally ASCII or ISO 8859-n) 	text/plain
.vsd 	Microsoft Visio 	application/vnd.visio
.wav 	Waveform Audio Format 	audio/x-wav
.weba 	WEBM audio 	audio/webm
.webm 	WEBM video 	video/webm
.webp 	WEBP image 	image/webp
.woff 	Web Open Font Format (WOFF) 	font/woff
.woff2 	Web Open Font Format (WOFF) 	font/woff2
.xhtml 	XHTML 	application/xhtml+xml
.xls 	Microsoft Excel 	application/vnd.ms-excel
.xlsx 	Microsoft Excel (OpenXML) 	application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
.xml 	XML 	text/xml
.xul 	XUL 	application/vnd.mozilla.xul+xml
.zip 	ZIP archive 	application/zip
.3gp 	3GPP audio/video container 	video/3gpp
.3g2 	3GPP2 audio/video container 	video/3gpp2
.7z 	7-zip archive 	application/x-7z-compressed"""

mime = {}
for line in data.split("\n"):
    elems = line.split("\t")
    try:
        mime[elems[0].replace(" ", "")] = elems[2].replace(" ", "")
    except:
        print(elems)
        continue


class MIME:

    mime = mime

    @classmethod
    def get_content_type(cls, extension):
        return cls.mime.get(extension)



from typing import List

from pydantic import BaseModel

from .StatusMessage import StatusMessage


class ResponseObject(BaseModel):
    """
    Representation of a successful completion response.

    Abstract, subclasses should instantiate this with their own type
    Subclasses
    ----------
    response.AnnotationsResponse
    response.AudioResponse
    response.ClassificationResponse
    response.StoredResponse
    response.TextsResponse

    Attributes
    ----------
    type (string, required on subclass): type of response
    warnings (List[StatusMessage], optional): messages describing any warnings on responses
    """

    type: str
    warnings: List[StatusMessage] = None



from typing import List

from pydantic import BaseModel, Field

from .. import ResponseObject


class ClassesResponse(BaseModel):
    """
    Classification object: classification and score (optional likelihood of classification)

    Attributes
    ----------
    class: (str, required) labelled class
    score: (int, optional) confidence score in class
    """

    class_field: str = Field(None, alias="class")
    score: int = None


class ClassificationResponse(ResponseObject):
    """
    Response encapsulating one or more classifications of the whole input message, optionally with confidence scores
    attached.

    Attributes
    ----------
    type (string, required): type of response
    warnings (List[StatusMessage], optional): messages describing any warnings on responses
    classes: (List[ClassesResponse], optional) : list of classications, zero or more allowed
    """

    type: str = "classification"
    classes: List[ClassesResponse] = None

    def auto_content(self):
        if self.warnings is not None:
            return self.dict()
        if len(self.classes) == 1:
            return self.classes[0].dict()
        return self.dict()["classes"]



from .Annotation import Annotation
from .Failure import Failure
from .Progress import Progress
from .Request import Request
from .Response import Response
from .ResponseObject import ResponseObject
from .StatusMessage import StatusMessage



from .ISO639 import ISO639
from .MIME import MIME
from .utils import (API_URL, CORPUS_DOWNLOAD_URL, LICENCE_URL, S3_UPLOAD_URL,
                    XML_UPLOAD_URL, get_argument_from_json, get_domain,
                    get_en_value, get_information, get_metadatarecord,
                    map_metadatarecord_to_result)



from .AnnotationsResponse import AnnotationsResponse
from .AudioResponse import AudioResponse
from .ClassificationResponse import ClassificationResponse
from .TextsResponse import TextsResponse


def get_response(response):
    try:
        response = response["response"]
        if response["type"] == "annotations":
            return AnnotationsResponse(**response)
        elif response["type"] == "audio":
            return AudioResponse(**response)
        elif response["type"] == "classification":
            return ClassificationResponse(**response)
        elif response["type"] == "texts":
            return TextsResponse(**response)
        else:
            raise ValueError(f"Response type {response['type']} not known.")
    except Exception as e:
        if e == ValueError:
            raise e
        print("Response: ", response)
        raise e
